/**
 * @file statsgen.h
 * @brief Statsgen class and all needed structures
 *
 * @author Jean-Baptiste Jorand
 * @author Yannick Bass
 * Copyright (c) 2018 Jorand Jean-baptiste Bass Yannick
 * All rights reserved.
 *
 * Please see the attached LICENSE file for additional licensing information.
 */


#ifndef STATSGEN_H
#define STATSGEN_H


#include <string>
#include <iostream>

#include "Policy.h"
#include "SecurityRules.h"
#include "Statistics.h"


struct PasswordStats {
	int pass_length = 0;
	std::string advancedmask_string = "";
	std::string simplemask_string = "";
	Policy pol;
};

/**
 * @brief Calcul and save all analysed statistics
 */
class Statsgen {
public:
	Statsgen(){}
	Statsgen(const std::string& name){ settings.filename = name; }

	int generate_stats();
	void print_stats() const;

	inline void setHiderare(const int& hr) { settings.hiderare = hr; }
	inline void setTop(const int& t) { settings.top = t; }
	inline void setRegex(const std::string& reg) { settings.regex_str = reg;	}
	inline void setWithcount(const bool& wc) { settings.withcount = wc; }
	inline void setLimitSimplemask(const int& limit) { settings.limitSimplemask = limit; }
	inline void setLimitAdvancedmask(const int& limit) { settings.limitAdvancedmask = limit; }
	inline void setNbThread(const int& nb) { settings.nbThread = nb; }
	void setSecurityRules(const uint& length, const uint& special, const uint& digit, const uint& upper, const uint& lower){
		settings.sr = { length, special, digit, upper, lower };
	}
	inline void setOutfile(const std::string& outfile) { settings.outfile_name = outfile; }
	inline void enableDebug() { settings.debug_enabled = true; }

	inline int getNbThreads() const { return settings.nbThread; }
	inline uint64_t getNbLines() const { return nblines; }
	inline uint64_t getProcessed() const { return processed; }
	inline const Statistics& getResults() const { return results; }
	inline bool allFinished() const { return finished; }
	inline bool allStarted() const { return started; }
	bool operator==(const Statsgen& other) const;

private:
	// results of the computation
	Statistics results;
	// stores the settings of the computation
	Settings settings;

	// nb lines processed
	uint64_t nblines = 0;
	// nb lines processed
	uint64_t processed = 0;
	// all threads have finished
	uint finished = false;
	// all threads have been started
	bool started = false;

	void handle_password(const std::string& password, const uint64_t& nbPasswords, Statistics& td) const;
	std::pair<uint, uint> get_masks(const std::string& password, PasswordStats& c) const;
};

#endif
