/**
 * @file utils.h
 * @brief useful functions
 *
 * @author Jean-Baptiste Jorand
 * @author Yannick Bass
 * Copyright (c) 2018 Jorand Jean-baptiste Bass Yannick
 * All rights reserved.
 *
 * Please see the attached LICENSE file for additional licensing information.
 */

#ifndef UTILS_STATSGEN_H
#define UTILS_STATSGEN_H

#include <map>
#include <iostream>
#include <fstream>
#include <unordered_map>
#include <iomanip>
#include <sstream>

typedef unsigned int uint;

inline float percentage(const float& num, const float& den){
	return 100 * num / den;
}

/**
 * @brief Order an unoredered map whatever the key's type
 * @param src: unoredered map than needed to be ordered
 * @return ordered map
 */
template<typename A, typename B>
std::multimap<B, A, std::greater<B>> flip_map(const std::unordered_map<A, B>& src){
	std::multimap<B, A, std::greater<B>> dst;
	for(std::pair<A, B> it : src){
		dst.insert(std::make_pair(it.second, it.first));
	}

	return dst;
}

/**
 * @brief Calculate the number of line in a file
 * @param filename: name of the file
 * @return number of line
 */
uint64_t nbline_file(const std::string& filename);

#endif