#include <iostream>

#include "Statistics.h"
#include "Utils.h"

using namespace std;

void minMax::updateMinMax(const minMax &m){
	mindigit = std::min(mindigit, m.mindigit);
	minlower = std::min(minlower, m.minlower);
	minupper = std::min(minupper, m.minupper);
	minspecial = std::min(minspecial, m.minspecial);
	maxdigit = std::max(maxdigit, m.maxdigit);
	maxlower = std::max(maxlower, m.maxlower);
	maxupper = std::max(maxupper, m.maxupper);
	maxspecial = std::max(maxspecial, m.maxspecial);
}

bool minMax::operator==(const minMax& m) const {
	return mindigit == m.mindigit
	&& minlower == m.minlower
	&& minupper == m.minupper
	&& minspecial == m.minspecial
	&& maxdigit == m.maxdigit
	&& maxlower == m.maxlower
	&& maxupper == m.maxupper
	&& maxspecial == m.maxspecial;
}

void Statistics::operator+=(const Statistics& other){
	*this = *this + other;
}

Statistics Statistics::operator+(const Statistics& other) const {
	Statistics td(*this);
	td.total_counter = total_counter + other.total_counter;
	td.total_filter = total_filter + other.total_filter;

	td.nb_secure_passwords += other.nb_secure_passwords;

	td.minMaxValue.updateMinMax(other.minMaxValue);


	for(std::pair<int, uint64_t> occ: other.length){
		td.length[occ.first] += occ.second;
	}

	for(std::pair<std::string, int> occ : other.charactersets){
		td.charactersets[occ.first] += occ.second;
	}

	for(std::pair<std::string, int> occ: other.simplemasks){
		td.simplemasks[occ.first] += occ.second;
	}

	for(std::pair<std::string, int> occ: other.advancedmasks){
		td.advancedmasks[occ.first] += occ.second;
	}
	return td;
}

bool Statistics::operator==(const Statistics& o) const {
	return total_counter == o.total_counter
	&& total_filter == o.total_filter
	&& nb_secure_passwords == o.nb_secure_passwords
	&& minMaxValue == o.minMaxValue
	&& length == o.length
	&& charactersets == o.charactersets
	&& simplemasks == o.simplemasks
	&& advancedmasks == o.advancedmasks;
}

std::ostream& operator<<(std::ostream& os, const Statistics& results){
	float perc = percentage(results.total_filter, results.total_counter);
	minMax m = results.minMaxValue;

	os << "Processed " << results.total_filter << " on " << results.total_counter << " passwords (" << perc << " %)" << endl;

	perc = percentage(results.nb_secure_passwords, results.total_counter);

	os << setw(42) << "--> " << results.nb_secure_passwords << " passwords (" << perc << " %) respect the security rules" << endl;
	os << endl;
	os << "min - max" << endl;
	os << setw(42)  << "digit:  "   << m.mindigit   << " - " << m.maxdigit   << endl;
	os << setw(42)  << "lower:  "   << m.minlower   << " - " << m.maxlower   << endl;
	os << setw(42)  << "upper:  "   << m.minupper   << " - " << m.maxupper   << endl;
	os << setw(42)  << "special:  " << m.minspecial << " - " << m.maxspecial << endl;
	return os;
}

void Statistics::show(std::ostream& os, const Settings& settings) const {
	os << *this << endl;
	os << endl << "Statistics relative to length: " << endl;
	showMap(os, length, settings.top, total_counter, settings.hiderare) << endl;

	os << endl << "Statistics relative to charsets: " << endl;
	showMap(os, charactersets, -1, total_counter, settings.hiderare) << endl;


	os << endl << "Statistics relative to simplemasks: " << endl;
	showMap(os, simplemasks, settings.top, total_counter, settings.hiderare) << endl;

	if (settings.limitSimplemask > 0) {
		os << endl;
		auto r = simplemasks.find("othermasks");
		if(r != simplemasks.end()){
			readResult(os, r->second, r->first, total_counter, settings.hiderare) << endl;
		}
	}


	os << endl << "Statistics relative to advancedmask: " << endl;
	showMap(os, advancedmasks, settings.top, total_counter, settings.hiderare) << endl;

	if (settings.limitAdvancedmask > 0) {
		os << endl;
		auto r = advancedmasks.find("othermasks");
		if(r != advancedmasks.end()){
			readResult(os, r->second, r->first, total_counter, settings.hiderare) << endl;
		}
	}

	if (! settings.outfile_name.empty()){
		locale::global(locale("C"));
		ofstream outfile_stream(settings.outfile_name);
		multimap<uint64_t, string, greater<uint64_t>> reverse = flip_map(advancedmasks);
		for(pair<uint64_t, string> it : reverse){
			if(it.second == "othermasks") continue;
			outfile_stream << it.second << "," << it.first << endl;
		}
		outfile_stream.close();
	}
}